#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Thu 06 May 2010 08:10:11 PM CST

''' address book '''

from system import getlist
from system import getunicode
from system import NASError
from system import sqlite, HOMEDB
from system import _all, _any

def getgroups(login=None, **kw):
    if login is None:
        raise NASError('Bad Request')

    login = getunicode(login)

    con = sqlite.connect(HOMEDB)
    cur = con.cursor()

    groups = []
    cur.execute(''' SELECT count() FROM friends WHERE owner = ?
                ''', (login,))
    counts = cur.fetchone()[0]
    groups.insert(0, {'name': '',
                      'comment': '',
                      'count': str(counts)})

    cur.execute(''' SELECT id, name, comment
                    FROM groups WHERE owner = ? ORDER BY name ASC
                ''', (login,))

    for group in cur.fetchall():
        (groupid, name, comment) = group
        cur.execute(''' SELECT count() FROM members WHERE groupid = ?
                    ''', (groupid,))
        count = cur.fetchone()[0]
        groups.append({'name': name.encode('utf8'),
                       'comment': comment.encode('utf8'),
                       'count': str(count)})
    con.close()
    return {'group': groups}

def addgroup(login=None, name=None, comment='', **kw):
    if not _all(login, name):
        raise NASError('Bad Request')

    login = getunicode(login)
    name = getunicode(name)
    comment = getunicode(comment)

    if not name:
        raise NASError('Bad Request')

    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    try:
        cur.execute(''' INSERT INTO groups (owner, name, comment)
                        VALUES (?, ?, ?)
                    ''', (login, name, comment))
    except sqlite.IntegrityError:
        con.close()
        raise NASError('Group Exists', name.encode('utf8'))
    con.commit()
    con.close()
    return getgroups(login)

def editgroup(login=None, name=None, newname=None, comment=None, **kw):
    if not _all(login, name):
        raise NASError('Bad Request')
    if not _any(newname, comment):
        raise NASError('Bad Request')

    login = getunicode(login)
    name = getunicode(name)

    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    if comment is not None:
        cur.execute(''' UPDATE groups SET comment = ?
                        WHERE owner = ? AND name = ?
                    ''', (comment, login, name))

    if newname is not None:
        newname = getunicode(newname)
        try:
            cur.execute(''' UPDATE groups SET name = ?
                            WHERE owner = ? AND name = ?
                        ''', (newname, login, name))
        except sqlite.IntegrityError:
            con.close()
            raise NASError('Group Exists', newname.encode('utf8'))

    con.commit()
    con.close()
    return {'addrbook': 'update group'}

def delgroup(login=None, name=None, **kw):
    ''' name can be multi '''
    if not _all(login, name):
        raise NASError('Bad Request')

    login = getunicode(login)
    names = getlist(name)

    con = sqlite.connect(HOMEDB)
    cur = con.cursor()

    for name in names:
        cur.execute(''' SELECT id FROM groups WHERE owner = ? AND name = ?
                    ''', (login, name))
        groupid = cur.fetchone()
        if groupid is None:
            continue
        cur.execute(''' DELETE FROM groups WHERE id = ? ''', groupid)
        cur.execute(''' DELETE FROM members WHERE groupid = ? ''', groupid)

    con.commit()
    con.close()
    return {'addrbook': 'update group'}

def getmembers(login=None, group=None, **kw):
    if login is None:
        raise NASError('Bad Request')

    login = getunicode(login)
    if group is not None and getunicode(group):
        return getgroupmembers(login, group)

    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    cur.execute(''' SELECT name, email, server, blog, phone, address, comment
                    FROM friends WHERE owner = ?
                    ORDER BY name ASC
                ''', (login,))
    members = []
    for member in cur.fetchall():
        member = [x.encode('utf8') for x in member]
        (name, email, server, blog, phone, address, comment) = member
        members.append({'name': name,
                        'email': email,
                        'server': server,
                        'blog': blog,
                        'phone': phone,
                        'address': address,
                        'comment': comment,
                        })
    con.close()
    return {'member': members}

def addmember(login=None, name=None, group=None, **kw):
    if not _all(login, name):
        raise NASError('Bad Request')

    items = ['email', 'server',
             'blog', 'phone', 'address', 'comment']

    login = getunicode(login)
    name = getunicode(name)
    lists = [login, name]

    for item in items:
        lists.append(getunicode(kw.get(item, '')))

    con = sqlite.connect(HOMEDB)
    cur = con.cursor()

    try:
        cur.execute(''' INSERT INTO friends (owner, name, email, server,
                            blog, phone, address, comment)
                        VALUES (?, ?, ?, ?, ?, ?, ?, ?)
                    ''', tuple(lists))
    except sqlite.IntegrityError:
        con.close()
        raise NASError('Member Exists', name.encode('utf8'))

    if group is not None:
        cur.execute(''' SELECT id FROM friends WHERE owner = ? AND name = ?
                    ''', (login, name))
        friendid = cur.fetchone()
        groups = getlist(group)
        for group in groups:
            if not group:
                continue
            cur.execute(''' SELECT id FROM groups WHERE owner = ? AND name = ?
                        ''', (login, group))
            groupid = cur.fetchone()
            if groupid is None:
                continue
            cur.execute(''' INSERT INTO members (groupid, friendid)
                            VALUES (?, ?)
                        ''', groupid + friendid)
    con.commit()
    con.close()
    return {'addrbook': 'add member'}

def editmember(login=None, name=None, **kw):
    if not _all(login, name):
        raise NASError('Bad Request')

    # if not _any(newname, email, server, blog, phone, address, comment):
    #     raise NASError('Bad Request')

    login = getunicode(login)
    name = getunicode(name)

    con = sqlite.connect(HOMEDB)
    cur = con.cursor()

    items = ['email', 'server', 'blog', 'phone', 'address', 'comment',
             'newname']
    for item in items:
        if kw.get(item) is not None:
            value = getunicode(kw.get(item))
            if item == 'newname':
                item = 'name'
            try:
                cur.execute(''' UPDATE friends SET %s = ?
                                WHERE owner = ? AND name = ?
                            ''' % item, (value, login, name))
            except sqlite.IntegrityError:
                con.commit()
                con.close()
                raise NASError('Member Exists', name)

    con.commit()
    con.close()
    return {'addrbook': 'update member'}

def delmember(login=None, name=None, **kw):
    ''' name can be multi '''

    if login is None or name is None:
        raise NASError('Bad Request')

    login = getunicode(login)
    names = getlist(name)

    con = sqlite.connect(HOMEDB)
    cur = con.cursor()

    for name in names:
        cur.execute(''' SELECT id FROM friends WHERE owner = ? AND name = ?
                    ''', (login, name))
        friendid = cur.fetchone()
        if friendid is None:
            continue
        cur.execute(''' DELETE FROM friends WHERE id = ? ''', friendid)
        cur.execute(''' DELETE FROM members WHERE friendid = ? ''', friendid)

    con.commit()
    con.close()
    return {'addrbook': 'update member'}

def getgroupmembers(login=None, group=None, **kw):
    if not _all(login, group):
        raise NASError('Bad Request')

    login = getunicode(login)
    group = getunicode(group)

    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    cur.execute(''' SELECT f.name, f.email, f.server, f.blog,
                        f.phone, f.address, f.comment
                    FROM friends as f, groups as g, members as m
                    WHERE g.owner = ?  AND g.name = ?
                        AND g.id = m.groupid AND m.friendid = f.id
                    ORDER BY f.name ASC
                ''', (login, group))
    members = []
    for member in cur.fetchall():
        keys = ['name', 'email', 'server', 'blog',
                'phone', 'address', 'comment']
        values = [x.encode('utf8') for x in member]
        newmember = {}
        for key in keys:
            newmember[key] = values.pop(0)
        members.append(newmember)
    con.close()
    return {'member': members}

def addgroupmember(login=None, group=None, friend=None, **kw):
    ''' friend can be multi '''
    if login is None or group is None or friend is None:
        raise NASError('Bad Request')

    login = getunicode(login)
    group = getunicode(group)
    friends = getlist(friend)

    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    cur.execute(''' SELECT id FROM groups WHERE owner = ? AND name = ?
                ''', (login, group))
    groupid = cur.fetchone()
    if groupid is None:
        con.close()
        raise NASError('Group Not Exists', group.encode('utf8'))

    for friend in friends:
        cur.execute(''' SELECT id FROM friends WHERE owner = ? AND name = ?
                    ''', (login, friend))
        friendid = cur.fetchone()
        if friendid is None:
            continue
        try:
            cur.execute(''' INSERT INTO members (groupid, friendid)
                            VALUES (?, ?)
                        ''', groupid + friendid)
        except sqlite.IntegrityError:
            continue

    con.commit()
    con.close()
    return {'addrbook': 'update group member'}

def delgroupmember(login=None, group=None, friend=None, **kw):
    ''' friend can be multi '''
    if login is None or group is None or friend is None:
        raise NASError('Bad Request')

    login = getunicode(login)
    group = getunicode(group)
    friends = getlist(friend)

    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    cur.execute(''' SELECT id FROM groups WHERE owner = ? AND name = ?
                ''', (login, group))
    groupid = cur.fetchone()
    if groupid is None:
        con.close()
        raise NASError('Group Not Exists', group.encode('utf8'))

    for friend in friends:
        cur.execute(''' SELECT id FROM friends WHERE owner = ? AND name = ?
                    ''', (login, friend))
        friendid = cur.fetchone()
        if friendid is None:
            continue
        cur.execute(''' DELETE FROM members WHERE groupid = ? AND friendid = ?
                    ''', groupid + friendid)

    con.commit()
    con.close()
    return {'addrbook': 'update group member'}

__all__ = ['getgroups', 'addgroup', 'editgroup', 'delgroup',
           'getmembers', 'addmember', 'editmember', 'delmember',
           'getgroupmembers', 'addgroupmember', 'delgroupmember']

# vim: set sta sw=4 et:
