#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Sun 27 Jun 2010 03:37:10 PM CST

''' Generate Thumbnail '''

import os
import stat
import errno
import fcntl
import magic
import shutil
import Image
import gst
import gobject
from JpegImagePlugin import RAWMODE

AUDIO_THUMBNAIL = '/var/www/nas/Applications/MyNAS/icons/audio_thumbnail.jpg'
BAD_THUMBNAIL = '/var/www/nas/Applications/MyNAS/icons/bad_thumbnail.jpg'

VIDEOS = ['application/ogg',
          'application/x-pn-realmedia',
          'application/x-shockwave-flash']

from system import _mkdir

def _unlink(name):
    try:
        mode = os.stat(name).st_mode
    except os.error as error:
        if error.errno == errno.ENOENT:
            return True

    if stat.S_ISDIR(mode):
        delete = os.rmdir
    else:
        delete = os.unlink

    try:
        delete(name)
    except os.error:
        return False

    return True

def _symlink(src, dst):
    try:
        _unlink(dst)
        os.symlink(src, dst)
    except os.error as error:
        # 'filesystem not support?'
        if error.errno == errno.EPERM:
            shutil.copy(src, dst)
            return True
        return False
    return True

def convert(filepath):
    try:
        mode = os.stat(filepath).st_mode
    except os.error as error:
        if error.errno == errno.ENOENT:
            return {}
        return {}
    if stat.S_ISLNK(mode):
        return convert(os.readlink(filepath))
    if not stat.S_ISREG(mode):
        return {}

    mime = magic.open(magic.MAGIC_MIME)
    if mime != None:
        mime.load()

    mimetype = mime.file(filepath).split(';')[0]

    # application/ogg
    # application/x-pn-realmedia
    # application/x-shockwave-flash
    if mimetype[:5] == 'video' or mimetype in VIDEOS:
        filetype = 'video'
        # generate thumbnail
        imagepath = '/tmp/%s.png' % os.getpid()
        result = Thumbnail(filepath, imagepath)
        mimetype = mime.file(imagepath).split(';')[0]
        if mimetype[:5] == 'image': # good
            filetype = 'video'
        elif result.has_audio:
            filetype = 'audio'
        else:
            filetype = 'none'
    elif mimetype[:5] == 'audio':
        filetype = 'audio'
    elif mimetype[:5] == 'image':
        filetype = 'image'
        imagepath = filepath
    else:
        filetype = 'none'

    _dir, name = os.path.split(filepath)
    wdir = os.path.join(_dir, '.webview')
    tdir = os.path.join(_dir, '.thumbnail')
    _mkdir(wdir)
    _mkdir(tdir)

    wname = os.path.join(wdir, name + '.jpg')
    tname = os.path.join(tdir, name + '.jpg')
    try:
        os.unlink(wname)
    except os.error:
        pass

    try:
        os.unlink(tname)
    except os.error:
        pass

    if filetype == 'video' or filetype == 'image':
        result = _convert(imagepath, wname, tname)
        if filetype == 'video':
            _unlink(wname)
            del result['.webview']
        if imagepath != filepath:
            _unlink(imagepath)
        return result

    if filetype == 'audio':
        thumbnail = AUDIO_THUMBNAIL
    elif filetype == 'none':
        thumbnail = BAD_THUMBNAIL

    _symlink(thumbnail, tname)

    return {'.thumbnail': thumbnail}

def _convert(filepath, wname, tname):

    flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC | os.O_SYNC
    wdesc = os.open(wname, flags, 0664)
    tdesc = os.open(tname, flags, 0664)
    fcntl.lockf(wdesc, fcntl.LOCK_EX)
    fcntl.lockf(tdesc, fcntl.LOCK_EX)

    try:
        img = Image.open(filepath)
        img.load()
    except IOError as error:
        if error.errno is not None:
            fcntl.lockf(wdesc, fcntl.LOCK_UN)
            fcntl.lockf(tdesc, fcntl.LOCK_UN)
            os.close(wdesc)
            os.close(tdesc)
            return {'.webview': BAD_THUMBNAIL,
                    '.thumbnail': BAD_THUMBNAIL}

    (width, height) = img.size
    scale = float(height) / width
    if scale < 0.75:
        wsize = [int(800 * x) for x in (1, scale)]
        tsize = [int(128 * x) for x in (1, scale)]
    elif scale < 1:
        wsize = [int(600 / x) for x in (scale, 1)]
        tsize = [int(128 * x) for x in (1, scale)]
    else:
        wsize = [int(600 / x) for x in (scale, 1)]
        tsize = [int(128 / x) for x in (scale, 1)]

    if 'transparency' in img.info or img.mode == 'RGBA':
        new = img.mode == 'RGBA' and img or img.convert('RGBA')
        img = Image.new('RGB', new.size)
        img.paste(new, (0, 0), new)
    elif img.mode not in RAWMODE:
        img = img.convert('RGB')

    img.thumbnail(wsize, Image.BILINEAR)
    wfile = os.fdopen(wdesc, 'w')
    try:
        img.save(wfile, 'JPEG', quality=50)
        wfile.flush()
    except OSError:
        fcntl.lockf(wdesc, fcntl.LOCK_UN)
        fcntl.lockf(tdesc, fcntl.LOCK_UN)
        wfile.close()
        os.close(tdesc)
        return {'.webview': BAD_THUMBNAIL,
                '.thumbnail': BAD_THUMBNAIL}

    # we have save webview, thumbnail shouldn't have any problem
    img.thumbnail(tsize, Image.BILINEAR)
    tfile = os.fdopen(tdesc, 'w')
    img.save(tfile, 'JPEG', quality=50)
    tfile.flush()

    os.fsync(wdesc)
    os.fsync(tdesc)
    fcntl.lockf(wdesc, fcntl.LOCK_UN)
    fcntl.lockf(tdesc, fcntl.LOCK_UN)
    wfile.close()
    tfile.close()

    return {'.webview': wname,
            '.thumbnail': tname}

class Thumbnail:

    def __init__(self, location, thumbnail, second=None):
        gobject.threads_init()

        # get abs path
        location = os.path.abspath(location)
        self.thumbnail = os.path.abspath(thumbnail)
        if not os.path.isfile(location):
            return None

        self.seeked = False
        self.playing = False
        self.has_video = False
        self.has_audio = False
        self.width = None
        self.height = None
        self.length = -1

        self.second = isinstance(second, (int, float)) and second or 0

        self.pipeline = gst.Pipeline()

        bus = self.pipeline.get_bus()
        bus.add_signal_watch()
        bus.connect('message', self.on_message)

        src = gst.element_factory_make('filesrc')
        src.set_property('location', location)
        decode = gst.element_factory_make('decodebin')
        decode.connect('new-decoded-pad', self.on_new_decoded_pad)
        decode.connect('no-more-pads', self.on_no_more_pads)

        # video
        ffmpeg = gst.element_factory_make('ffmpegcolorspace')
        # ffmpeg = gst.element_factory_make('videoscale')
        pngenc = gst.element_factory_make('pngenc')
        filesink = gst.element_factory_make('filesink')
        filesink.set_property('location', self.thumbnail)
        self.pipeline.add(src, decode, ffmpeg, pngenc, filesink)

        src.link(decode)
        gst.element_link_many(ffmpeg, pngenc, filesink)

        self.video = ffmpeg.get_pad('sink')
        self.video.connect("notify::caps", self.on_notify_caps)

        self.pipeline.set_state(gst.STATE_PAUSED)
        self.mainloop = gobject.MainLoop()
        self.mainloop.run()

    def on_new_decoded_pad(self, decode, pad, last):
        caps = pad.get_caps()
        # name = caps[0].get_name()
        if 'video' in caps.to_string():
            self.has_video = True
            if not self.video.is_linked():
                pad.link(self.video)
            self.pipeline.set_state(gst.STATE_PLAYING)
            self.playing = True
        if 'audio' in caps.to_string():
            self.has_audio = True

    def on_no_more_pads(self, decode):
        if not self.playing:
            self.pipeline.set_state(gst.STATE_PLAYING)

    def on_message(self, bus, message):
        if message.type in [gst.MESSAGE_EOS, gst.MESSAGE_ERROR]:
            self.pipeline.set_state(gst.STATE_NULL)
            self.mainloop.quit()
            #if not self.has_video:
            #    os.unlink(self.thumbnail)
        #elif message.type == gst.MESSAGE_TAG:
        #    tags = message.parse_tag()
        #    if 'duration' in tags:
        #        self.length = tags['duration']
        if not self.seeked and self.second:
            position = self.second * gst.SECOND
            flag = gst.SEEK_FLAG_FLUSH
            if self.length > position:
                self.seeked = True
                self.pipeline.seek_simple(gst.FORMAT_TIME, flag, position)
            elif self.length != -1:
                self.seeked = True
                self.pipeline.seek_simple(gst.FORMAT_TIME, flag, self.length)

    def on_notify_caps(self, pad, arg):
        caps = pad.get_negotiated_caps()
        if not caps:
            return None
        if 'video' in caps.to_string():
            self.width = caps[0]['width']
            self.height = caps[0]['height']
        query = gst.query_new_duration(gst.FORMAT_TIME)
        if pad.get_peer().query(query):
            gst.FORMAT_TIME, self.length = query.parse_duration()

__all__ = ['convert']

# vim: set sta sw=4 et:
