#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Mon 09 May 2011 10:50:29 AM CST

COMPANY = 'Works System, Inc.'
DEVICE = 'iSharing'
VERSION = '2.0'
USERAGENT = '%s - %s - %s' % (COMPANY, DEVICE, VERSION)

import socket
import urllib
import httplib
# from nas.system import get_name_value

from nas.system import get_name_value
from nas.system import set_name_value
from nas.system import NASError
from nas.system import decrypt

HOST = 'members.dyndns.org'
PATH = '/nic/update'
DEVICE = '/etc/nas/device'

import struct
aton = lambda x: struct.unpack('!I', socket.inet_aton(x))[0]
# mask = lambda x: int('1' * int(x) + '0' * (32 - int(x)), 2)
# PRIVATE = ((aton('10.0.0.0') & mask(8), mask(8)),
#            (aton('172.16.0.0') & mask(12), mask(12)),
#            (aton('192.168.0.0') & mask(16), mask(16)))
PRIVATE = ((167772160, 4278190080), (2886729728, 4293918720), (3232235520, 4294901760))
def isprivate(ipaddr):
    try:
        address = aton(ipaddr)
    except socket.error:
        return None

    for gateway, netmask in PRIVATE:
        if (address & netmask) == (gateway & netmask):
            return True

    return False

def update(eth0='', force=False, wildcard='NOCHG', mx='NOCHG', backmx='NOCHG'):
    names = {'dyndns_host': '',
             'dyndns_user': '',
             'dyndns_eth0': '',}
    get_name_value(DEVICE, names)
    if not eth0 or not names['dyndns_host'] or not names['dyndns_user']:
        return False

    if eth0 == names['dyndns_eth0']:
        if not force:
            return True
    else:
        names['dyndns_eth0'] = eth0

    if isprivate(eth0):
        myip = ''
    else:
        myip = eth0

    values = {'hostname': names['dyndns_host'],
              'myip': myip,
              'wildcard': wildcard,
              'mx': mx,
              'backmx': backmx,
              }

    path = '%s?%s' % (PATH, urllib.urlencode(values))
    # print 'path = %s' % path
    conn = httplib.HTTPConnection(HOST)
    conn.putrequest('GET', path)
    conn.putheader('Authorization', 'Basic %s' % names['dyndns_user'])
    conn.putheader('User-Agent', USERAGENT)

    status, content = None, None
    try:
        conn.endheaders()
        resp = conn.getresponse()
        status = resp.status
        content = resp.read()
        resp.close()
    except socket.error:
        pass
    finally:
        conn.close()

    if status == 200 and content:
        import re
        names['dyndns_last'] = re.sub(r'[\t\n\x0b\x0c\r]', '', content)

    set_name_value(DEVICE, names)
    return status, content

def getdyndns(**kw):
    '''
    arguments: None
    returns: hostname (list), the hostname(s) of setting
             lastresp, last response from server when update dyndns
    '''
    names = {'dyndns_host': '',
             'dyndns_last': '',}
    get_name_value(DEVICE, names)

    dyndns = {'hostname': names['dyndns_host'].split(','),
              'lastresp': names['dyndns_last'],
              }
    return {'dyndns': dyndns}

def setdyndns(hostname, username=None, password=None, session=None, **kw):
    '''
    arguments: hostname (list)
               username,
               password, (encrypt)
    returns: refer getdyndns
    '''
    if session is None:
        raise NASError('Bad Request')

    if isinstance(hostname, basestring):
        hostname = [hostname]
    names = {'dyndns_host': ','.join(hostname)}

    if username and password and session:
        auth = '%s:%s' % (username, decrypt(session, password))
        names['dyndns_user'] = auth.encode('base64').strip()

    if set_name_value(DEVICE, names):
        names = {'dyndns_eth0': ''}
        get_name_value(DEVICE, names)
        update(names['dyndns_eth0'], force=True)

    return getdyndns()

__all__ = ['update', 'getdyndns', 'setdyndns']

if __name__ == '__main__':
    # update()
    print isprivate('222.30.47.86')
    print isprivate('10.10.10.10')

# vim: set sta sw=4 et:
