#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Wed 17 Nov 2010 08:11:29 PM CST

''' RSA '''

import struct
from subprocess import Popen, PIPE
from cStringIO import StringIO

def get_number(data, length):
    number = 0L
    while length:
        bits = data.read(1)
        number = number << 8 | struct.unpack('B', bits)[0]
        length -= 1
    return number

def get_length(data):
    length = get_number(data, 1)
    if length == 0x81:
        length = get_number(data, 1)
    elif length == 0x82:
        length = get_number(data, 2)
    elif length == 0x83:
        length = get_number(data, 3)
    return length

def get_oid(data, amount):
    def get_id(data):
        value = 0
        while True:
            number = get_number(data, 1)
            value = (value << 7) + (number & 0x7f)
            if not number >> 7:
                break
        return value

    string = ''
    while data.tell() < amount:
        oid = get_id(data)
        if not string:
            string += '.'.join([str(x) for x in divmod(oid, 40)])
        else:
            string += '.%s' % oid
    return string

def parse_sequence(data, sequence, end):
    while data.tell() < end:
        tag = get_number(data, 1)
        length = get_length(data)
        if tag == 0x02: # integer
            sequence.append(get_number(data, length))
        elif tag == 0x05: # null
            # length should be 0
            data.read(length)
        elif tag == 0x06: # OID
            get_oid(data, length + data.tell())
        elif tag == 0x03:
            get_number(data, 1)     # discard
            parse_sequence(data, sequence, length + data.tell() - 1)
        elif tag == 0x30:
            parse_sequence(data, sequence, length + data.tell())
        else:
            raise Exception('RSA Error')
    assert data.tell() == end

def parse_private(data):
    sequence = []
    tag = get_number(data, 1)
    length = get_length(data)
    assert tag == 0x30
    parse_sequence(data, sequence, length + data.tell())
    return sequence

def generate(rsabit):
    args = ['openssl', 'genrsa', str(rsabit)]
    popen = Popen(args, stdout=PIPE, stderr=PIPE)
    popen.wait()
    popen.stderr.close()
    if popen.returncode != 0:
        return generate(rsabit)

    content = ''
    while True:
        line = popen.stdout.readline().strip()
        if line == '-----BEGIN RSA PRIVATE KEY-----':
            continue
        elif line == '-----END RSA PRIVATE KEY-----':
            popen.stdout.close()
            break
        content += line

    key = StringIO(content.decode('base64'))
    sequence = parse_private(key)

    public = 0x10001
    if public in sequence[1:]:
        index = sequence.index(public, 1)
        modulus = sequence[index-1]
        if sequence[index+1:]:
            private = sequence[index+1]
    return public, modulus, private

__all__ = ['generate']

# vim: set sta sw=4 et:
