#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Tue 03 Aug 2010 01:22:15 PM CST

import os
import re
import time

import errno
import fcntl

from cStringIO import StringIO

from system import get_name_value
from system import POPEN
from system import getvendor
from system import getpid
from network import gethwaddr
from system import getvendor

import _ped
import parted

def getversion(url):
    info = {'hardware': '', 'software': ''}
    get_name_value('/etc/nas/version', info)
    output = StringIO()
    info['vendor'] = getvendor()
    output.write('hardware=%s, software=%s, gui=%s\r\n' % (info['hardware'], info['software'], info['vendor']))
    info.clear()
    return output

def getinternalstorage(url):
    output = StringIO()
    try:
        device = parted.Device('/dev/sda')
    except _ped.IOException:
        return output
    model = device.model
    size = device.length * device.sectorSize
    output.write('model=%s, size=%s bytes\r\n' % (model, size))
    return output

def getexternalstorage(url):
    output = StringIO()
    for disk in os.listdir('/dev'):
        if disk not in ['sda', 'sdb'] and disk[:2] == 'sd' and len(disk) == 3:
            try:
                device = parted.Device('/dev/%s' % disk)
            except _ped.IOException:
                continue
            model = device.model
            size = device.length * device.sectorSize
            output.write('model=%s, size=%s bytes\r\n' % (model, size))
    return output

def getesatastorage(url):
    output = StringIO()
    try:
        device = parted.Device('/dev/sdb')
    except _ped.IOException:
        return output
    model = device.model
    size = device.length * device.sectorSize
    output.write('model=%s, size=%s bytes\r\n' % (model, size))
    return output

def getmacaddr(url):
    output = StringIO()
    output.write('%s\r\n' % gethwaddr())
    return output

#def sethwaddr(hwaddr=None, url):
#    output = StringIO()
#    file = open('/proc/cpuinfo')
#    cpuinfo = file.read()
#    file.close()
#    if 'plx7821' not in cpuinfo:
#        output.write('only support plx7821\r\n')
#        return output
#
#    b = '([0-9a-fA-F]{2})'
#    c = '[:-]?'
#    a = '^' + (b + c) * 5 + b + '$'
#    import re
#    result = re.findall(a, str(hwaddr))
#    if not result:
#        output.write('bad request\r\n')
#        return output
#    data = ''.join(map(lambda x: chr(int(x, 16)), result[0]))
#    mtd = '/dev/mtdblock1'
#    dd = os.open(mtd, os.O_RDWR)
#    os.lseek(dd, 0xa40000 - 0x1000, os.SEEK_SET)
#    os.write(dd, data)
#    os.write(dd, '\xff' * (0x800 - 6))
#    os.write(dd, data)
#    os.write(dd, '\xff' * (0x800 - 6))
#    os.close(dd)
#    output.write('set hwaddr to %s\r\n' % ':'.join(result[0]))
#    return output

def gethwclock(url):
    output = StringIO()
    (error, pipe) = POPEN('hwclock')
    if error != 0:
        output.write('[error]: %s' % pipe.read())
    else:
        output.write('[hwclock] %s' % pipe.read())
    pipe.close()
    return output

def getseckey(url):
    import nas.info
    output = StringIO()
    output.write('seckey = %s' % nas.info.getseckey())
    return output

def sethwclock(url):
    result = re.findall('(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})$', url)
    output = StringIO()
    if not result:
        output.write('[hwclock]: Bad Request\r\n')
        return output
    date = '%s-%s-%s %s:%s:%s' % result[0]
    command = ['su', '-c', 'hwclock --set --date="%s"' % date]
    (error, pipe) = POPEN(command)
    if error != 0:
        output.write('[error]: %s' % pipe.read())
    else:
        output.write('[hwclock] set hardware clock')
    pipe.close()
    return output

def blinkled(url):
    output = StringIO()
    if not os.path.exists('/sys/class/leds'):
        return output
    for led in os.listdir('/sys/class/leds'):
        fd = os.open('/sys/class/leds/%s/trigger' % led, os.O_WRONLY)
        os.write(fd, 'timer')
        os.close(fd)
        output.write('[led] %s = blink\r\n' % led)
    fd = os.open('/sys/class/leds/system/trigger', os.O_WRONLY)
    os.write(fd, 'default-on')
    os.close(fd)
    fd = os.open('/sys/class/leds/system/trigger', os.O_WRONLY)
    os.write(fd, 'timer')
    os.close(fd)
    return output

def turnonled(url):
    output = StringIO()
    if not os.path.exists('/sys/class/leds'):
        return output
    for led in os.listdir('/sys/class/leds'):
        fd = os.open('/sys/class/leds/%s/trigger' % led, os.O_WRONLY)
        os.write(fd, 'default-on')
        os.close(fd)
        output.write('[led] %s = on\r\n' % led)
    fd = os.open('/sys/class/leds/system/trigger', os.O_WRONLY)
    os.write(fd, 'timer')
    os.close(fd)
    fd = os.open('/sys/class/leds/system/trigger', os.O_WRONLY)
    os.write(fd, 'default-on')
    os.close(fd)
    return output

def turnoffled(url):
    output = StringIO()
    if not os.path.exists('/sys/class/leds'):
        return output
    for led in os.listdir('/sys/class/leds'):
        fd = os.open('/sys/class/leds/%s/trigger' % led, os.O_WRONLY)
        os.write(fd, 'none')
        os.close(fd)
        output.write('[led] %s = off\r\n' % led)
    fd = os.open('/sys/class/leds/system/trigger', os.O_WRONLY)
    os.write(fd, 'timer')
    os.close(fd)
    fd = os.open('/sys/class/leds/system/trigger', os.O_WRONLY)
    os.write(fd, 'none')
    os.close(fd)
    return output

def _getdate(date=None):
    if date is None:
        date = time.time()
    ms = int(repr(date).split('.')[1])
    string = time.strftime('%Y-%m-%d %H:%M:%S', time.gmtime(date))
    string += '.%08d' % ms
    return string

BUTTONPID = '/var/run/nas-button.pid'
def startbuttontest(url):
    pid = getpid(BUTTONPID)
    if pid is not None:
        os.kill(pid, 15)
    output = StringIO()

    output.write('[%s] stop normal button function\r\n' % _getdate())

    record = os.open('/var/log/nas-buttontest.log', os.O_WRONLY | os.O_CREAT)
    try:
        fcntl.lockf(record, fcntl.LOCK_EX | fcntl.LOCK_NB)
    except IOError as error:
        if error.errno in [errno.EAGAIN, errno.EACCES]:
            output.write('is anotherone using this function?\r\n')
        os.close(record)
        return output
    else:
        os.close(record)
        if os.fork() == 0:
            (error, pipe) = POPEN('nas-buttontest')
            pipe.close()
            os._exit(0)
        return output

def getbuttontest(url):
    output = StringIO()
    try:
        fp = open('/var/log/nas-buttontest.log', 'r+')
    except IOError as error:
        if error.errno == errno.ENOENT:
            output.write('did you begin button test?\r\n')
        return output
    fcntl.lockf(fp, fcntl.LOCK_EX)
    output.write(fp.read())
    fcntl.lockf(fp, fcntl.LOCK_UN)
    fp.close()
    return output

def stopbuttontest(url):
    POPEN('nas-buttons')[1].close()
    output = StringIO()
    output.write('[%s] start normal button funciton\r\n' % _getdate())
    try:
        os.unlink('/var/log/nas-buttontest.log')
    except os.error:
        pass
    return output

def testonly(url):
    functions = {
        'get:version': getversion,
        'get:internal_diskcapacity': getinternalstorage,
        'mpget:macaddress': getmacaddr,
        'mpget:external_usbcapacity': getexternalstorage,
        'mpget:external_esatacapacity': getesatastorage,
        'mpset:clock=': sethwclock,
        'mpget:hwclock': gethwclock,
        'mpget:seckey': getseckey,
        'mpset:testledson': turnonled,
        'mpset:testledsoff': turnoffled,
        'mpset:testledsblink': blinkled,
        'mpset:testbuttonbegin': startbuttontest,
        'mpset:testbuttonstatus': getbuttontest,
        'mpset:testbuttonend': stopbuttontest,
    }
    url = url.lower()
    keys = functions.keys()
    keys.sort(cmp=lambda x, y: len(x) - len(y), reverse=True)
    for key in keys:
        if url.startswith(key):
            function = functions.get(key)
            return function(url)

    output = StringIO()
    output.write('Bad Request\r\n')
    return output

__all__ = ['testonly']

# vim: set sta sw=4 et:
