#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Mon 05 Jul 2010 03:54:17 PM CST

''' USBCopy '''

import os
import time
import errno
import shutil
import fcntl
import leds
from system import POPEN
from system import _mkdir

DEVICES = '/media'
USBNAME = 'USBCOPY'
PUBLIC = '/home/Public'
USBCOPYLOCK = '/var/run/usbcopy.lock'


def getusbdir():
    dirs = []
    for i in os.listdir(DEVICES):
        if i.startswith(USBNAME):
            try:
                os.rmdir(os.path.join(DEVICES, i))
            except OSError as error:
                if error.errno != errno.EBUSY:
                    continue
            else:
                continue

            if not dirs:
                dirs.append(i)
            elif len(i) <= len(dirs[0]) and i <= dirs[0]:
                dirs.insert(0, i)
    if not dirs:
        leds.control(leds.USBCOPY_LED, leds.OFF)
        return None

    return os.path.join(DEVICES, dirs[0])

def usbcopy(**kw):
    usbdir = getusbdir()

    lock = os.open(USBCOPYLOCK, os.O_WRONLY | os.O_CREAT)
    try:
        fcntl.lockf(lock, fcntl.LOCK_EX | fcntl.LOCK_NB)
    except IOError as error:
        fcntl.lockf(lock, fcntl.LOCK_UN)
        if error.errno == errno.EACCES or error.errno == errno.EAGAIN:
            return None

    usbdirvfs = os.statvfs(usbdir)
    maxblocks = 32 * (1024**3) / usbdirvfs.f_frsize
    if (usbdirvfs.f_blocks - usbdirvfs.f_bfree) > maxblocks:
        leds.control(leds.USBCOPY_LED, leds.OFF)
        return None

    _mkdir(PUBLIC)

    homedirvfs = os.statvfs(PUBLIC)
    space = homedirvfs.f_bfree * homedirvfs.f_frsize
    if (usbdirvfs.f_blocks - usbdirvfs.f_bfree) * usbdirvfs.f_frsize > space:
        leds.control(leds.USBCOPY_LED, leds.OFF)
        return None

    dstdir = os.path.join(PUBLIC, time.strftime('UPLOAD_%Y%m%d_%H%M'))
    if os.path.exists(dstdir):
        if os.path.isdir(dstdir):
            shutil.rmtree(dstdir, True)
        else:
            os.unlink(dstdir)
    # os.mkdir(dstdir)

    # blink
    leds.control(leds.USBCOPY_LED, leds.BLINK)

    shutil.copytree(usbdir, dstdir, True)
    (error, status) = POPEN('umount %s' % usbdir)
    status.close()

    # close
    leds.control(leds.USBCOPY_LED, leds.OFF)

    fcntl.lockf(lock, fcntl.LOCK_UN)
    os.close(lock)
    os.unlink(USBCOPYLOCK)

    return True

if __name__ == '__main__':
    usbcopy()

# vim: set sta sw=4 et:
